// graffities.c

/* ************************************************************** */
/*
    graffities
    paint the screen (open for XSendEvent)
    (ideas and code used from lasertraq-0.2)
    Copyright (C) 2007  Fred Koegler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "graffities.h"
#include "brushes.h"
#include "palettes.h"
#include "graffities_screen.c"

static char short_options[] = "hV:f:r:";
static struct option long_options[] =
{
  { "help", no_argument, NULL, 'h' },
  { "version", no_argument, NULL, 'V' },
  { "fullscreen", no_argument, NULL, 'f'},
  { "resolution", required_argument, NULL, 'r'},
  { 0, 0, 0, 0 }
};

/* ************************************************************** */
void print_usage(void)
{
  fprintf(stderr,"%s - %s - %s\n",PROG,PROGRAM,VERSION);
  fprintf(stderr,"This software comes with ABSOLUTELY NO WARRANTY\n");
  fprintf(stderr,"This software is free software, and you are welcome to redistribute it\n");
  fprintf(stderr,"under certain conditions\n");
  fprintf(stderr,"usage: %s [-hVf] [-r <wdith>x<height>]\n",PROG);
  fprintf(stderr,"\t-h --help\t\tdisplay this help screen\n");
  fprintf(stderr,"\t-V --version\t\tdisplay version and exit\n");
  fprintf(stderr,"\t-f --fullscreen\tRun fullscreen\n");
  fprintf(stderr,"\t-r --resolution <wdith>x<height>\tResolution <wdith>x<height>\n");
}

/* ************************************************************** */
int parameters(struct VideoPaint *vp, int *argc, char *argv[])
{
  int done = 0;
  int c;
  FILE *preffile;

  while (! done)
  {
    c = getopt_long(*argc,argv,short_options,long_options,NULL);
    switch (c) {
    case 'h' :
      print_usage();
      exit(0);
      break;
    case 'V' :
      fprintf(stdout,"%s - %s - %s\n",prog,program,version);
      exit(0);
      break;
    case 'f':
      vp->screen.sdl_flags |= SDL_FULLSCREEN;
      break;
    case 'r':
      if (!optarg)
      {
        fprintf(stderr,"Error: resolution (<WIDTH>x<HEIGHT>)\n");
        exit(1);
      }
      else
      {
        sscanf(optarg,"%dx%d",&vp->screen.width,&vp->screen.height);
        if (vp->screen.width <= 0 || vp->screen.height <= 0)
        {
          fprintf(stderr,"Error: Invalid resolution.\n");
          exit(1);
        }
      }
      break;
    case '?' :
      fprintf(stderr,"Error: invalid option or ambiguous argument\n");
      print_usage();
      exit(1);
      break;
    case EOF :
      done = 1;
      break;
    default :
      print_usage();
      exit(1);
    };
  }
}

/* ************************************************************** */
int video_screen_mode(struct VideoPaint *vp)
{
  int error = 0;

  if(!(vp->screen.display = SDL_SetVideoMode(
    vp->screen.width,vp->screen.height,vp->screen.depth,vp->screen.sdl_flags)))
  {
    fprintf(stderr,"sdl-display open error: \"%s\"\n",SDL_GetError());
    error = -2;
  }
  else
  {
    SDL_EventState(SDL_ACTIVEEVENT, SDL_IGNORE);
    vp->screen.pixelbuf = (Uint8 *) vp->screen.display->pixels;
    error = 0;
  }
  return error;
}

/* ************************************************************** */
int video_screen_init(struct VideoPaint *vp)
{
  int error = 0;

  if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
  {
    fprintf(stderr,"Error: sdl-init-video \"%s\"\n", SDL_GetError());
    error = -1;
  }
  else
  {
    atexit(SDL_Quit);

    error = video_screen_mode(vp);
  }
  return error;
}

/* ************************************************************** */
int screen_draw_frame(struct VideoPaint *vp)
{
  if (SDL_MUSTLOCK (vp->screen.display))
    if (SDL_LockSurface(vp->screen.display) < 0)
    {
      fprintf(stderr,"Error: Can't lock Surface: %s\n",SDL_GetError());
      return -1;
    }

  if (vp->mouse_button)
  {
    screen_draw_line(vp,vp->old_x,vp->old_y,vp->mouse_x,vp->mouse_y);
    vp->old_x = vp->mouse_x;
    vp->old_y = vp->mouse_y;
    screen_draw_brush(vp,vp->mouse_x,vp->mouse_y);
  }

//  screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
//    0,0,vp->screen.width,vp->screen.height);

  if (vp->draw_gadgets) screen_draw_gadgets(vp);

  if (SDL_MUSTLOCK (vp->screen.display))
    SDL_UnlockSurface(vp->screen.display);

  SDL_UpdateRect(vp->screen.display,0,0,0,0);

  return 0;
}

/* ************************************************************** */
void MOUSEMOTION(struct VideoPaint *vp, int x, int y)
{
  vp->mouse_x = x;
  vp->mouse_y = y;
  if (! vp->mouse_button)
  {
    vp->old_x = vp->mouse_x;
    vp->old_y = vp->mouse_y;
  }
}

/* ************************************************************** */
void MOUSEBUTTONUP(struct VideoPaint *vp)
{
  vp->mouse_button = 0;
  vp->draw_gadgets = 1;
}

/* ************************************************************** */
void MOUSEBUTTONDOWN(struct VideoPaint *vp)
{
  vp->mouse_button = 1;
  if ((vp->mouse_x <= vp->gadgetsize)
  && (vp->mouse_y <= (vp->gadgets * vp->gadgetsize)))
  {
    int gadget;
    gadget = vp->mouse_y / vp->gadgetsize;
    switch (gadget)
    {
    case 0:
      SDL_FillRect(vp->screen.display, &vp->screen.rect, 0);
      vp->draw_gadgets = 1;
      break;
    case 1:
      vp->brush++;
      if (vp->brush >= max_brushes) vp->brush = 0;
      vp->draw_gadgets = 1;
      break;
    case 2:
      vp->brushscale++;
      if (vp->brushscale > max_brushscale) vp->brushscale = 1;
      vp->draw_gadgets = 1;
      break;
    case 3:
      vp->bluronly = 1 - vp->bluronly;
      vp->draw_gadgets = 1;
      break;
    case 4:
      vp->palette++;
      if (vp->palette >= max_palettes) vp->palette = 0;
      vp->draw_gadgets = 1;
      break;
    case 5:
      screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
        0,0,vp->screen.width,vp->screen.height);
      screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
        0,0,vp->screen.width,vp->screen.height);
      vp->draw_gadgets = 1;
      break;
    default:
      break;
    }
  }
}

/* ************************************************************** */
int main(int argc, char *argv[])
{
  static struct VideoPaint vp;
  SDL_Event event;
  int mx,my;

  vp.mousedown = 0;

  vp.screen.width = 1024;
  vp.screen.height = 768;
  vp.screen.sdl_flags = SDL_SWSURFACE | SDL_HWPALETTE;

  vp.draw_gadgets = 1;
  vp.brush = 0;
  vp.brushsize = 16;
  vp.brushscale = max_brushscale-1;
  vp.bluronly = 0;
  vp.palette = 0;
  vp.gadgets = 6;
  vp.gadgetsize = 3 * vp.brushsize + 4;
  vp.mouse_button = 0;

  parameters(&vp,&argc,argv);

  vp.screen.rect.x = 0;
  vp.screen.rect.y = 0;
  vp.screen.rect.w = vp.screen.width;
  vp.screen.rect.h = vp.screen.height;
  vp.screen.size = vp.screen.width * vp.screen.height;
  vp.screen.depth = 8;

  if (video_screen_init(&vp))
  {
    fprintf(stderr,"Error: sdl-init-screen\n");
    return (-1);
  }

  char *title = (char*)malloc (strlen(PROGRAM)+strlen(PROG)+strlen(VERSION)+10); 
  sprintf (title,"%s - %s - %s",PROG,PROGRAM,VERSION);
  SDL_WM_SetCaption(title, NULL);
  free (title);

  int SDL_Loop = 1;
  int old_palette = -1;
  while (SDL_Loop)
  {
    if (vp.palette != old_palette)
    {
      SDL_SetColors(vp.screen.display,(SDL_Color*)&palettes[vp.palette],0,256);
      old_palette = vp.palette;
    }
    screen_draw_frame(&vp);

    SDL_Delay(20);

    while (SDL_PollEvent(&event))
    {
      switch (event.type)
      {
      case SDL_QUIT:
        SDL_Loop = 0;
        break;

      case SDL_KEYUP:
        break;

      case SDL_KEYDOWN:
        switch (event.key.keysym.sym)
        {
// ...
//        case SDLK_F12:
//          if (vp.screen.sdl_flags & SDL_FULLSCREEN)
//            vp.screen.sdl_flags |= SDL_FULLSCREEN;
//          else
//            vp.screen.sdl_flags &= ~SDL_FULLSCREEN;
//          if (video_screen_mode(&vp))
//          {
//            fprintf(stderr,"Error: sdl-init-screen\n");
//            return (-1);
//          }
//          break;
        case SDLK_ESCAPE:
          SDL_Loop = 0;
          break;
        case SDLK_SPACE:
          SDL_FillRect(vp.screen.display,&vp.screen.rect,0);
          screen_draw_gadgets(&vp);
          break;
        default:
          break;
        }
        break; 

      case SDL_MOUSEMOTION:
        MOUSEMOTION(&vp,(int)event.motion.x,(int)event.motion.y);
        break;

      case SDL_MOUSEBUTTONUP:
        MOUSEBUTTONUP(&vp);
        break;

      case SDL_MOUSEBUTTONDOWN:
        MOUSEBUTTONDOWN(&vp);
        break;

      default:
        fprintf(stderr,"Error: Unknown SDL event received ?!\n");
        break;
      }
    }
  }

  SDL_ShowCursor(SDL_ENABLE);

  exit(0);
}
