// graffities_screen.c

/* ************************************************************** */
void screen_blur_rect(Uint8 *buffer, int width, int x1, int y1, int x2, int y2)
{
  int x_count, y_count;
  int tmp;

  for (y_count=y1+2; y_count < y2-2; y_count++)
    for (x_count=x1+2; x_count < x2-2; x_count++)
    {
      tmp = (int) buffer[y_count*width+x_count-1];
      tmp += (int) buffer[y_count*width+x_count+1];
      tmp += (int) buffer[(y_count-1)*width+x_count];
      tmp += (int) buffer[(y_count-1)*width+x_count-1];
      tmp += (int) buffer[(y_count-1)*width+x_count+1];
      tmp += (int) buffer[(y_count+1)*width+x_count];
      tmp += (int) buffer[(y_count+1)*width+x_count-1];
      tmp += (int) buffer[(y_count+1)*width+x_count+1];
      tmp += (int) buffer[y_count*width+x_count-2];
      tmp += (int) buffer[y_count*width+x_count+2];
      tmp += (int) buffer[(y_count-2)*width+x_count];
      tmp += (int) buffer[(y_count-2)*width+x_count-2];
      tmp += (int) buffer[(y_count-2)*width+x_count+2];
      tmp += (int) buffer[(y_count+2)*width+x_count];
      tmp += (int) buffer[(y_count+2)*width+x_count-2];
      tmp += (int) buffer[(y_count+2)*width+x_count+2];
      tmp /= 16;
      buffer[y_count*width+x_count] = (Uint8) tmp;
    }
}

/* ************************************************************** */
void screen_blur_circ(Uint8 *buffer, int width, int x1, int y1, int x2, int y2)
{
  int x_count, y_count;
  double r, xm, ym;
  int tmp;

  r = fabs(x2-x1)+fabs(y2-y1)/4.0;
  xm = x1+(x2-x1)/2.0;
  ym = y1+(y2-y1)/2.0;
  for (y_count=y1+2; y_count < y2-2; y_count++)
    for (x_count=x1+2; x_count < x2-2; x_count++)
      if (sqrt(pow((x_count-xm),2.0)+pow((y_count-ym),2.0)) <= r)
      {
        tmp = (int) buffer[y_count*width+x_count-1];
        tmp += (int) buffer[y_count*width+x_count+1];
        tmp += (int) buffer[(y_count-1)*width+x_count];
        tmp += (int) buffer[(y_count-1)*width+x_count-1];
        tmp += (int) buffer[(y_count-1)*width+x_count+1];
        tmp += (int) buffer[(y_count+1)*width+x_count];
        tmp += (int) buffer[(y_count+1)*width+x_count-1];
        tmp += (int) buffer[(y_count+1)*width+x_count+1];
        tmp += (int) buffer[y_count*width+x_count-2];
        tmp += (int) buffer[y_count*width+x_count+2];
        tmp += (int) buffer[(y_count-2)*width+x_count];
        tmp += (int) buffer[(y_count+2)*width+x_count];
        tmp /= 12;
        buffer[y_count*width+x_count] = (Uint8) tmp;
      }
}

/* ************************************************************** */
void screen_draw_brush(struct VideoPaint *vp, int x, int y, int scale = 1, int not_in_gadgets = 1)
{
  scale = (not_in_gadgets ? vp->brushscale : scale );

  if (not_in_gadgets && vp->gadgets
  && (x <= (vp->gadgetsize + vp->brushsize * scale * 3/5))
  && (y <= (vp->gadgets * vp->gadgetsize + vp->brushsize * scale * 3/5)))
    return;

  if ((! not_in_gadgets)
  || ((x > (vp->brushsize * scale * 3/5))
  && (x <= (vp->screen.width - vp->brushsize * scale * 3/5))
  && (y > (vp->brushsize * scale * 3/5))
  && (y <= (vp->screen.height - vp->brushsize * scale * 3/5))))
  {
    int lines,cols,i,j,x_n,y_n;

    x_n = x - vp->brushsize * scale/2;
    y_n = y - vp->brushsize * scale/2;

    if ((! not_in_gadgets) || (! vp->bluronly))
    {
      Uint8 *buffer = &vp->screen.pixelbuf[y_n*vp->screen.width+x_n];
      for (lines = 0; lines < vp->brushsize; lines++)
      {
        for (cols = 0; cols < vp->brushsize; cols++)
          for (i = 0; i < scale; i++)
            for (j = 0; j < scale; j++)
              buffer[i*vp->screen.width+cols*scale+j] |=
                brushes[vp->brush][lines*vp->brushsize+cols];
        buffer += vp->screen.width * scale;
      }
    }
    screen_blur_circ(vp->screen.pixelbuf,vp->screen.width,
      x_n,y_n,x_n+vp->brushsize*scale,y_n+vp->brushsize*scale);
  }
}

/* ************************************************************** */
void screen_draw_line(struct VideoPaint *vp, register int x1, register int y1, register int x2, register int y2)
{
  register int deltaX, deltaY;	
  register int addX, addY;
  register int counter, error;

  addX = 1;
  addY = 1;

  deltaX = x2 - x1;
  deltaY = y2 - y1;


  if(deltaX < 0)
  {
    addX = -addX;
    deltaX = -deltaX;
  }
  if(deltaY < 0)
  {
    addY = -addY;
    deltaY = -deltaY;
  }

  if (deltaX == 0)
  {
    for(counter = deltaY; counter > 0; counter--)
    {
      screen_draw_brush(vp,x1,y1);
      y1 += addY;
    }
    return;
  }

  if (deltaY == 0)
  {
    for(counter = deltaX; counter > 0; counter--)
    {
      screen_draw_brush(vp,x1,y1);
      x1 += addX;
    }
    return;
  }
  if (deltaX > deltaY)
  {
    // calc initial error-value
    error = deltaX>>1;
    for(counter = deltaX; counter > 0; counter--)
    {
      // adjust error-value
      error += deltaY;
      // set pixel with clipping
      screen_draw_brush(vp,x1,y1);
      // if error is out of range, go one line further up or down
      if (error >= deltaX)
      {
        error -= deltaX;
        y1 += addY;
      }
      x1 += addX;
    }
  }
  else
  {
    // calc initial error-value
    error = deltaY>>1;
    for (counter = deltaY; counter >= 0; counter--)
    {
      // adjust error-value
      error += deltaX;
      // set pixel with clipping
      screen_draw_brush(vp,x1,y1);
      // if error is out of range, go one line further left or right
      if (error >= deltaY)
      {
        error -= deltaY;
        x1 += addX;
      }
      y1 += addY;
    }
  }
}

/* ************************************************************** */
void screen_draw_gadgets(struct VideoPaint *vp)
{
  int x,y,dx,dy,scry,scryw;
  SDL_Rect rect,rect1;

  if (vp->gadgets)
  {
    rect.x = 0;
    rect.y = 0;
    rect.w = vp->gadgetsize + vp->brushsize/* * vp->brushscale*/;
    rect.h = vp->gadgets * vp->gadgetsize + vp->brushsize/* * vp->brushscale*/;
    SDL_FillRect(vp->screen.display, &rect, 0);

    rect.w = vp->gadgetsize;
    rect.h = vp->gadgets * vp->gadgetsize;
    SDL_FillRect(vp->screen.display,&rect,255);

    rect.x++;
    rect.y++;
    rect.w-=2;
    rect.h = vp->gadgetsize-2;

    for(y = 0; y < vp->gadgets; y++)
    {
      scry = y * vp->gadgetsize;
      scryw = scry * vp->screen.width;
      rect.y = scry + 1;
      SDL_FillRect(vp->screen.display,&rect,0);
      switch (y)
      {
      case 0 :
        break;
      case 1 :
        screen_draw_brush(vp,(rect.w)/2,scry+(rect.h)/2,2,0);
        break;
      case 2 :
        rect1.x = rect.x + 1 + (rect.w-1)/2 - 1 - 2 * vp->brushscale;
        rect1.y = rect.y + 1 + (rect.h-1)/2 - 1 - 2 * vp->brushscale;
        rect1.w = 2 + 4 * vp->brushscale;
        rect1.h = 2 + 4 * vp->brushscale;
        SDL_FillRect(vp->screen.display,&rect1,255);
        for(dy = 0; dy < vp->brushscale; dy++)
          screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
            1,scry+1,vp->gadgetsize-1,scry+vp->gadgetsize-1);
        break;
      case 3 :
        rect1.x = rect.x + 1;
        rect1.y = rect.y + 1;
        rect1.w = rect.w - 2;
        rect1.h = rect.h - 2;
        SDL_FillRect(vp->screen.display,&rect1,255*vp->bluronly);
        rect1.x = rect.x + 1 + (rect.w-1)/2 - 1 - 2 * vp->brushscale;
        rect1.y = rect.y + 1 + (rect.h-1)/2 - 1 - 2 * vp->brushscale;
        rect1.w = 2 + 4 * vp->brushscale;
        rect1.h = 2 + 4 * vp->brushscale;
        SDL_FillRect(vp->screen.display,&rect1,255*(1-vp->bluronly));
        for(dy = 0; dy < vp->brushscale; dy++)
          screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
            1,scry+1,vp->gadgetsize-1,scry+vp->gadgetsize-1);
        break;
      case 4 :
        for(dy = 0; dy < 2; dy++)
          for(dx = 0; dx < 2; dx++)
          {
            rect1.x = rect.x + 1 + dx * (rect.w-1)/2;
            rect1.y = rect.y + 1 + dy * (rect.h-1)/2;
            rect1.w = (rect.w-1)/2;
            rect1.h = (rect.h-1)/2;
            SDL_FillRect(vp->screen.display,&rect1,(dx+dy*2+1)*64-1);
          }
        screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
          1,scry+1,vp->gadgetsize-1,scry+vp->gadgetsize-1);
        break;
      case 5 :
        for(dy = 0; dy < 2; dy++)
          for(dx = 0; dx < 2; dx++)
          {
            rect1.x = rect.x + 1 + 2 + dx * (2+rect.w-1)/2;
            rect1.y = rect.y + 1 + 2 + dy * (2+rect.h-1)/2;
            rect1.w = (rect.w-1)/2 - 4;
            rect1.h = (rect.h-1)/2 - 4;
            SDL_FillRect(vp->screen.display,&rect1,(dx+dy*2+1)*64-1);
          }
          for(dy = 0; dy < 4; dy++)
            screen_blur_rect(vp->screen.pixelbuf,vp->screen.width,
              1,scry+1,vp->gadgetsize-1,scry+vp->gadgetsize-1);
        break;
      }
    }
    vp->draw_gadgets = 0;
  }
}
