#!/bin/sh 
# the next line restarts using wish \
exec wish "$0" "$@"
# ######################################################## #
#
# convert "POVRay color_map saved by Gimp"
#
# to "C rgb structure" that can included in palettes.h
#
# ######################################################## #
#
# input:
#
#/* color_map file created by the GIMP */
#/* http://www.gimp.org/               */
#color_map {
#	[0.000000 color rgbt <0.000000, 0.000000, 0.000000, 0.000000>]
#	[0.264608 color rgbt <1.000000, 1.000000, 1.000000, 0.000000>]
#	[0.502504 color rgbt <1.000000, 1.000000, 1.000000, 0.000000>]
#	[0.502504 color rgbt <1.000000, 1.000000, 1.000000, 0.000000>]
#	[0.601002 color rgbt <0.500000, 0.500000, 0.500000, 0.000000>]
#	[0.637730 color rgbt <0.000000, 0.000000, 0.000000, 0.000000>]
#	[0.637730 color rgbt <0.000000, 0.000000, 0.000000, 0.000000>]
#	[0.700343 color rgbt <0.445000, 0.164600, 0.088879, 0.000000>]
#	[0.744574 color rgbt <0.890000, 0.329199, 0.177758, 0.000000>]
#	[0.744574 color rgbt <0.890000, 0.329199, 0.177758, 0.000000>]
#	[0.767761 color rgbt <0.945000, 0.664600, 0.088879, 0.000000>]
#	[0.786311 color rgbt <1.000000, 1.000000, 0.000000, 0.000000>]
#	[0.786311 color rgbt <1.000000, 1.000000, 0.000000, 0.000000>]
#	[0.829920 color rgbt <0.955000, 0.787219, 0.000000, 0.000000>]
#	[0.891486 color rgbt <0.910000, 0.574437, 0.000000, 0.000000>]
#	[0.891486 color rgbt <0.910000, 0.574437, 0.000000, 0.000000>]
#	[0.944908 color rgbt <0.818636, 0.351187, 0.074185, 0.000000>]
#	[1.000000 color rgbt <1.000000, 1.000000, 1.000000, 0.000000>]
#} /* color_map */
#
# output:
#
# {
#  {  0,  0,  0},
#...
#  {255,255,255}
# }
#

global workdir inifilename
set workdir [ file dirname $argv0 ]
cd $workdir
set workdir [ pwd ]

set help "Usage:\n${argv0} <filename of POVRay color_map by Gimp>\n"

if { $argc != 1 } then {
  exit
  puts stderr $help
}
set filename [ lindex $argv 0 ]

set regexp_vector {^[\t\[ ]+([\.0-9]+)[ a-z\<]+([\.0-9]+)[, ]+([\.0-9]+)[, ]+([\.0-9]+)[, ]+([\.0-9]+)[\>\] ]+$}

set regexp_vector_test {^[\t\[ ]+[\.0-9]+[ a-z\<]+[\.0-9]+[, ]+[\.0-9]+[, ]+[\.0-9]+[, ]+[\.0-9]+[\>\] ]+$}

  if { [ file exists $filename ] } then {
    if { [ catch {open [file join [file nativename $filename]] r} fid ] } then {
      puts stderr $help
      puts stderr "Could not open for reading\n$fid"
    } else {
      set i 0
      set k {}
      set ca 0
      puts ""
      puts " \{ // color_map '${filename}'"
      while { ! [ eof $fid ] } {
        set line [ gets $fid ]
#        puts "line:'${line}'"
        if { [ string length "${line}" ] > 1 } then {
          if { [ regexp $regexp_vector $line ] } then {
            regexp $regexp_vector $line rgbt c r g b t
            set ia [ expr $ca * 255.0 ]
            set ie [ expr $c * 255.1 ]
            while { $i < $ie } {
              if { $i < 255 } then {
                set k ","
              } else {
                set k ""
              }
              set ri [ expr int(255.0*($ra+double($r-$ra)*($i-$ia)/($ie-$ia))) ]
              set gi [ expr int(255.0*($ga+double($g-$ga)*($i-$ia)/($ie-$ia))) ]
              set bi [ expr int(255.0*($ba+double($b-$ba)*($i-$ia)/($ie-$ia))) ]
              puts "    { $ri, $gi, $bi }$k"
              incr i
            }
            set ca $c
            set ra $r
            set ga $g
            set ba $b
            set ta $t
          }
        }
      }
      puts " \}"
      close $fid
    }
  } else {
    puts stderr $help
    puts stderr "Could not find file for reading\n$filename"
  }
